getROC = function(score, response){
  dataset = data.frame(score, response)
  dataset = dataset[order(score,decreasing = T),]
  dataset = transform(
    dataset,
    true_positive = cumsum(response) / sum(response),
    false_positive = cumsum(!response) / sum(!response),
    identity = seq(from = 0, to = 1, length = nrow(dataset))
  )
  return(dataset)
}

getAUC = function(rocs,...){
  if(class(rocs) != "list"){
    rocs = list(model = rocs)
  }
  extra_rocs = list(...)
  for(model in names(extra_rocs)){
    rocs[[model]] = extra_rocs[[model]]
  }
  suppressWarnings({
    dataset = map2_dfr(names(rocs), rocs, function(model, values){
      data.frame(
        model = model,
        AUC = sum( ( values$true_positive[-1] + values$true_positive[-nrow(values)] ) / 2 * diff(values$false_positive))
      )
    })
  })
  
  return(dataset)
}

plotROC = function(rocs = list(),...,show_auc = T){
  
  if(class(rocs) != "list"){
    rocs = list(model = rocs)
  }
  extra_rocs = list(...)
  for(model in names(extra_rocs)){
    rocs[[model]] = extra_rocs[[model]]
  }
  
  ## Formato tidy
  dataset = map2_dfr(names(rocs), rocs, function(model,values){
    values$model = model
    return(values)
  })
  
  # graficar
  gr = ggplot(dataset,aes(false_positive, true_positive,color = model)) + 
    geom_line(size = 1.5) + 
    geom_line(aes(x,y),data.frame(x = 0:1, y = 0:1),inherit.aes=F,size = 1.5)
  
  if(show_auc){
    auc_data = getAUC(rocs)
    auc_data$AUC = round(auc_data$AUC,2)
    auc_data$ACU = round(map_dbl(rocs, function(x) mean((round(x$score)==1) == x$response)),2)
    gr = gr + 
      annotation_custom(tableGrob(auc_data,rows=NULL), xmin=0.5, xmax=1, ymin=0, ymax=0.5)
  }
  return(gr)
}
